@@ -29,12 +29,14 @@ try:
     import userprofile
     import dirmonitor
     import mozilla_bookmarks
+    import debuglog
 except:
     from sabayon import util
     from sabayon import config
     from sabayon import userprofile
     from sabayon import dirmonitor
     from sabayon import mozilla_bookmarks
+    from sabayon import debuglog
 
 class file_state:
     (UNKNOWN,
@@ -55,6 +57,7 @@ sabayon_mandatory_pref_rel_path = os.path.join(firefox_rel_path, "sabayon-mandat
 sabayon_bookmark_rel_path = os.path.join(firefox_rel_path, "sabayon-bookmarks.html")
 sabayon_mandatory_bookmark_rel_path = os.path.join(firefox_rel_path, "sabayon-mandatory-bookmarks.html")
 
+# FIXME: these should be defined one place; see mozilla_bookmarks.py
 LOG_OPERATION           = 0x00001
 LOG_CHANGE              = 0x00002
 LOG_IGNORED_CHANGE      = 0x00004
@@ -67,10 +70,10 @@ LOG_DATA                = 0x00100
 LOG_VERBOSE             = 0x10000
 
 def dprint(mask, fmt, *args):
-    util.debug_print(util.DEBUG_MOZILLASOURCE, fmt % args, mask)
-
-def dwarn(fmt, *args):
-    print >> sys.stderr, "WARNING " + fmt % args
+    # FIXME: before debuglog was introduced, we could use the mask to filter
+    # which messages to log.  Now we don't use it anymore.  Is it still useful?
+    # If you change this, synchronize it with mozilla_bookmarks.py
+    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_MOZILLA_SOURCE, fmt % args)
 
 bookmark_exclude_attrs = [re.compile("^id$"),
                           re.compile("^last_"),
@@ -204,12 +207,12 @@ class MozillaDelegate(userprofile.SourceDelegate):
 
     def handle_change(self, change):
         rel_path = change.get_id()
+        dprint (LOG_CHANGE, "handle_change: rel_path='%s'", rel_path)
         #
         # INI File
         #
         if self.is_ini_file(rel_path):
-            dprint(LOG_CHANGE, "%s ini file: %s",
-                   dirmonitor.event_to_string(change.event), rel_path)
+            dprint(LOG_CHANGE, "%s ini file: %s", dirmonitor.event_to_string(change.event), rel_path)
             self.load_profiles_ini()
 
         #
@@ -217,8 +220,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         #
         profile = self.is_profile_file(rel_path)
         if profile:
-            dprint(LOG_CHANGE, "%s profile file: %s",
-                   dirmonitor.event_to_string(change.event), rel_path)
+            dprint(LOG_CHANGE, "%s profile file: %s", dirmonitor.event_to_string(change.event), rel_path)
 
             profile_file = profile.add_file(rel_path)
             profile_file_type = profile_file.get_type()
@@ -402,7 +404,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
 
         # --------------------
         if sabayon_pref_rel_path in pref_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_pref_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_pref_rel_path)
             self.source.storage.extract(sabayon_pref_rel_path, self.home_dir, True)
             apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_pref_rel_path)
             apply_pref.read()
@@ -410,7 +412,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
             apply_pref = None
 
         if sabayon_mandatory_pref_rel_path in pref_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_pref_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_mandatory_pref_rel_path)
             self.source.storage.extract(sabayon_mandatory_pref_rel_path, self.home_dir, True)
             mandatory_apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_mandatory_pref_rel_path)
             mandatory_apply_pref.read()
@@ -420,7 +422,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         # --------------------
 
         if sabayon_bookmark_rel_path in bookmark_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_bookmark_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_bookmark_rel_path)
             self.source.storage.extract(sabayon_bookmark_rel_path, self.home_dir, True)
             apply_bookmark = BookmarksFile(self.home_dir, sabayon_bookmark_rel_path)
             apply_bookmark.read()
@@ -428,7 +430,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
             apply_bookmark = None
 
         if sabayon_mandatory_bookmark_rel_path in bookmark_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_bookmark_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_mandatory_bookmark_rel_path)
             self.source.storage.extract(sabayon_mandatory_bookmark_rel_path, self.home_dir, True)
             mandatory_apply_bookmark = BookmarksFile(self.home_dir, sabayon_mandatory_bookmark_rel_path)
             mandatory_apply_bookmark.read()
@@ -710,8 +712,7 @@ class JavascriptPrefsFile(FirefoxProfileFile):
                 type  = match.group(1)
                 key   = match.group(2)
                 value = match.group(3)
-                dprint(LOG_PARSE, "(%d:%d) key='%s' value='%s'" %
-                       (match.start(), match.end(), key, value))
+                dprint(LOG_PARSE, "(%d:%d) key='%s' value='%s'", match.start(), match.end(), key, value)
                 self.prefs[key] = JavascriptPreference(type, key, value)
                 start = match.end()
             else:
@@ -727,7 +728,7 @@ class JavascriptPrefsFile(FirefoxProfileFile):
         keys = self.prefs.keys()
         keys.sort()
         for key in keys:
-            dprint(LOG_, "%s=%s" % (key, self.prefs[key]))
+            dprint(LOG_VERBOSE, "%s=%s", key, self.prefs[key])
 
 
 # ------ Class FirefoxProfile ------
@@ -829,8 +830,7 @@ class FirefoxProfilesIni:
 
 
     def read(self):
-        dprint(LOG_OPERATION, "FirefoxProfilesIni.read() path = %s",
-               self.get_full_path(self.rel_path))
+        dprint(LOG_OPERATION, "FirefoxProfilesIni.read() path = %s", self.get_full_path(self.rel_path))
         self.profiles = {}
 
         try:
@@ -1117,11 +1117,11 @@ def get_type_from_path(rel_path):
 
 def cat_file(path):
     if os.path.isfile(path):
-        dprint(LOG_FILE_CONTENTS, "==== %s ====" % path)
+        dprint(LOG_FILE_CONTENTS, "==== %s ====", path)
         for line in open(path):
             dprint(LOG_FILE_CONTENTS, line.rstrip())
     else:
-        dprint(LOG_FILE_CONTENTS, "WARNING, does not exist ==== %s ====" % path)
+        dprint(LOG_FILE_CONTENTS, "WARNING, does not exist ==== %s ====", path)
 
 
 
